import unittest
from revarie import Revarie
import numpy as np

class TestRevarie(unittest.TestCase):
    def test_init(self):
        """
        Test if any errors in initialization of Revarie instance
        """
        x = np.random.uniform(-1,1,(10,2))
        m = lambda h : np.sqrt(h)

        r = Revarie(x, 0, 1, m)

        self.assertTrue(r.mu == 0)

    def test_trivial(self):
        """
        Make sure field generation in trivial case is correct
        """
        x = np.random.uniform(-1,1, (5,2))
        m = lambda h : 0

        r = Revarie(x, 1, 0, m)

        self.assertTrue(np.allclose(r.genf(), 1))

    def test_shape(self):
        """
        Make sure length of generated field data array is same length of x
        """
        x = np.random.uniform(-1,1, (10,2))
        m = lambda h : 0

        r = Revarie(x, 1, 0, m)
        self.assertTrue(r.genf().size, x.shape[0])

    def test_cov(self):
        """
        Test generation of covariance matrix for trivial case
        """
        x = np.linspace(0,10,10)
        m = lambda h : 1

        r = Revarie(x, 1, 1, m)
        self.assertTrue(np.allclose(r.cov, np.eye(10)))





from revarie import Revarie
from revarie import Variogram
from revarie import fvariogram
import numpy as np
import time
import datetime
from .output import write
from pathlib import Path

def bench_variogram(tlimit = 15, path = "."):
    """
    Run timing benchmark test for variogram class. Results are formatted and
    printed to a file named "variogram_timing###.dat".

    Parameters
    ----------
    tlimit : float
        Approximate wall time limit for tests to run.
    path : str, path-like
        Path for results file to be printed to
    """
    tot_time = time.time()

    n = 50
    ns = []
    ts = []
    while time.time() - tot_time < tlimit:
        start = time.time()
        ns.append(n)
        x = np.random.uniform(0,1,n)
        f = np.random.uniform(0,1,n)

        Variogram(x,f)

        ts.append(time.time() - start)
        n = int(n*1.2)

    t = 0
    fname = Path("variogram_timing" + str(t).zfill(3) + ".dat")
    while (Path(path) / fname).is_file():
        t += 1
        fname = Path("variogram_timing" + str(t).zfill(3) + ".dat")
    write(np.asarray(ns),np.asarray(ts),Path(path) / fname,"Variogram", "1-D")

def bench_revarie(tlimit = 15, path = "."):
    """
    Run timing benchmark test for revarie class. Results are formatted and
    printed to a file named "revarie_timing###.dat".

    Parameters
    ----------
    tlimit : float
        Approximate wall time limit for tests to run.
    path : str, path-like
        Path for results file to be printed to
    """
    tot_time = time.time()

    nug = 0
    sill = 1
    rang = 0.3
    v = fvariogram("func", "sph", [nug, sill, rang])

    mu = 1

    n = 50
    ns = []
    ts = []
    while time.time() - tot_time < tlimit:
        start = time.time()
        ns.append(n)
        x = np.random.uniform(0,1,n)

        Revarie(x, mu, sill, v).genf()

        ts.append(time.time() - start)
        n = int(n*1.2)

    t = 0
    fname = Path("revarie_timing" + str(t).zfill(3) + ".dat")
    while (Path(path) / fname).is_file():
        t += 1
        fname = Path("revarie_timing" + str(t).zfill(3) + ".dat")
    write(np.asarray(ns),np.asarray(ts),Path(path) / fname,"Revarie", "1-D")

def suite(tlimit = 30, path = "."):
    """
    Run timing benchmark test for both the revarie and variogram classes.
    Results are formatted and printed to a file named "revarie_timing###.dat".
    Each benchmark is run with equal time division for each object.

    Parameters
    ----------
    tlimit : float
        Approximate wall time limit for tests to run.
    path : str, path-like
        Path for results files to be printed to
    """
    bench_variogram(tlimit/2, path = path)
    bench_revarie(tlimit/2, path = path)









import numpy as np
from scipy.optimize import curve_fit
import platform, subprocess
from datetime import datetime, date

def get_processor_info():
    """
    Simple function to get computer information output as a string
    """
    if platform.system() == "Windows":
        r = platform.processor()
    elif platform.system() == "Linux":
        command = "lscpu"
        r = subprocess.check_output(command, shell=True).strip()
    return r.decode("utf-8")


def write(n, t, fname, typ, notes=False):
    """
    Write results from benchmarking test to a file, also fits timing data to
    an equation of the form: time = k*(n)**p.

    Parameters
    ----------
    n : numpy array
        Array containing number of data points used for each test simulation
    t : numpy array
        Array containing wall time of each test simulation
    fname : str, path-like
        String or path object containing path and file name for test results
        to be printed
    typ : str
        "variogram" or "revarie", for which object it being
        benchmarked. Can be abbreviated, written directly to test results file
    notes : str
        Any extra information to be printed in results file
    """
    out = open(fname, "w")
    #Write CPU information
    out.write("% --- CPU info\n")
    out.write(get_processor_info() + "\n\n")

    #Write testing information
    out.write("% --- Test Info\n")
    out.write("Test Date/Time:".ljust(24) + str(datetime.now()) + "\n")
    out.write("Test Type:".ljust(24) + typ + "\n")
    mn = "%.2E"%n.min()
    mx = "%.2E"%n.max()
    out.write("Nrange:".ljust(24) + mn + "-" + mx + "\n\n")
    if notes:
        out.write("Notes:".ljust(24) + notes + "\n")

    #Report fit to exponential of form t=k*n^p
    out.write("% --- Results Summary\n")
    out.write("Total Runtime:".ljust(24) + str(t.sum()) + " s\n")
    a_est = np.log(t[0]/t[-1])/np.log(n[0]/n[-1])
    k_est = t[-1]/n[-1]**a_est
    (a,k), _ = curve_fit(lambda n, a, k : k*n**a,
                         xdata = n,
                         ydata = t,
                         p0 = (a_est, k_est))
    out.write("Fitted k:".ljust(24) + "%.4E\n"%k)
    out.write("Fitted a:".ljust(24) + "%.4E\n"%a)
    out.write("Predicted 1e5 Runtime:".ljust(24) + "%.4E s\n"%(k*1e5**a))


    #Write results from test
    out.write("\n\n% --- Timing Data\n")
    out.write("n             Wall Time [s]\n")
    for x1, x2 in zip(n, t):
        out.write(("%.5E"%x1).ljust(14))
        out.write("%.10E\n"%x2)
